<?php

namespace Litecms\Leasing\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Leasing\Forms\Lease as LeaseForm;
use Litecms\Leasing\Http\Requests\LeaseResourceRequest;
use Litecms\Leasing\Http\Resources\LeaseResource;
use Litecms\Leasing\Http\Resources\LeasesCollection;
use Litecms\Leasing\Models\Lease;
use Litecms\Leasing\Actions\LeaseAction;
use Litecms\Leasing\Actions\LeaseActions;


/**
 * Resource controller class for lease.
 */
class LeaseResourceController extends BaseController
{

    /**
     * Initialize lease resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = LeaseForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.leasing.modules'), 'leasing', guard_url('leasing'));
            return $next($request);
        });
    }

    /**
     * Display a list of lease.
     *
     * @return Response
     */
    public function index(LeaseResourceRequest $request)
    {
        $request = $request->all();
        $page = LeaseActions::run('paginate', $request);

        $data = new LeasesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('leasing::lease.names'))
            ->view('leasing::lease.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display lease.
     *
     * @param Request $request
     * @param Model   $lease
     *
     * @return Response
     */
    public function show(LeaseResourceRequest $request, Lease $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LeaseResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('leasing::lease.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('leasing::lease.show')
            ->output();
    }

    /**
     * Show the form for creating a new lease.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LeaseResourceRequest $request, Lease $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LeaseResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('leasing::lease.name'))
            ->view('leasing::lease.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new lease.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LeaseResourceRequest $request, Lease $model)
    {
        try {
            $request = $request->all();
            $model = LeaseAction::run('store', $model, $request);
            $data = new LeaseResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('leasing::lease.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('leasing/lease/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/leasing/lease'))
                ->redirect();
        }

    }

    /**
     * Show lease for editing.
     *
     * @param Request $request
     * @param Model   $lease
     *
     * @return Response
     */
    public function edit(LeaseResourceRequest $request, Lease $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LeaseResource($model);
        // return view('leasing::lease.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('leasing::lease.name'))
            ->view('leasing::lease.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the lease.
     *
     * @param Request $request
     * @param Model   $lease
     *
     * @return Response
     */
    public function update(LeaseResourceRequest $request, Lease $model)
    {
        try {
            $request = $request->all();
            $model = LeaseAction::run('update', $model, $request);
            $data = new LeaseResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('leasing::lease.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('leasing/lease/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('leasing/lease/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the lease.
     *
     * @param Model   $lease
     *
     * @return Response
     */
    public function destroy(LeaseResourceRequest $request, Lease $model)
    {
        try {

            $request = $request->all();
            $model = LeaseAction::run('destroy', $model, $request);
            $data = new LeaseResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('leasing::lease.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('leasing/lease/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('leasing/lease/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
